from .exceptions import APIException, JSONDecodeError, NoTokenProvided, ResponseError, RequestError
from .cattle import Environment

__title__ = "pyranch"
__version__ = "0.0.5"
__author__ = "Federico Torresan"


import requests
import json

from pyranch.storage_drivers import StorageDriver
from pyranch.volumes import Volume

try:
    # Python 3
    from urllib.parse import urljoin
except (ImportError) as e:
    # Python 2
    from urlparse import urljoin
from .exceptions import JSONDecodeError, RequestError, ResponseError
from .hosts import Host
from .stacks import Stack
from .services import Service
from .subscribe import Subscribe


class Environment(object):
    def __init__(self, url, access_key, secret_key, project_id=None, port=80, api_version='v2-beta'):
        endpoint = urljoin(url + ':' + str(port), api_version + '/projects/' + project_id + '/')
        self.project_id = project_id
        self.endpoint = endpoint
        self.auth = (access_key, secret_key)
        self.__dict__ = self.request(endpoint[:-1], 'GET')
        self.endpoint = endpoint
        self.auth = (access_key, secret_key)

        # Readonly fields
        self.project_data = None

    def __getattr__(self, name):
        if self.project_data is None:
            if self.project_id:
                self.project_data = self.request(self.endpoint[:-1], 'GET')
            else:
                # Create project object for list fetch
                self.project_data = {}
        return self.project_data.get('name') or None

    def __call__(self):
        print(json.dumps(self.__dict__, indent=4))

    def __str__(self):
        return b'<{:s} at {:#x}>'.format(type(self).__name__, id(self))

    def __unicode__(self):
        return '<{:s} at {:#x}>'.format(type(self).__name__, id(self))

    def __set_content_type(self, headers, ctype):
        headers.update({'content-type': ctype})

    def __get(self, url, params, data, headers):
        return requests.get(url, auth=self.auth, params=params, headers=headers)

    def __post(self, url, params, data, headers):
        self.__set_content_type(headers, 'application/json')
        return requests.post(url, auth=self.auth, data=json.dumps(data), params=params, headers=headers)

    def __put(self, url, params, data, headers):
        self.__set_content_type(headers, 'application/json')
        return requests.put(url, auth=self.auth, data=json.dumps(data), params=params, headers=headers)

    def __delete(self, url, params, data, headers):
        self.__set_content_type(headers, 'application/x-www-form-urlencoded')
        return requests.delete(url, auth=self.auth, params=params, headers=headers)

    def __request(self, url, method, params, data, headers=None):
        headers = headers or {}

        METHODS = {
            'get': self.__get,
            'post': self.__post,
            'put': self.__put,
            'delete': self.__delete
        }

        request_method = METHODS[method.lower()]
        url = urljoin(self.endpoint, url)

        return request_method(url, params=params, data=data, headers=headers)

    def request(self, url, method, params=None, data=None):
        params = params or {}
        data = data or {}

        response = self.__request(url, method, params, data)

        if response.status_code == 204:
            json_response = ''
        else:
            try:
                json_response = response.json()
            except ValueError:
                raise JSONDecodeError()

            if not response.ok:
                if response.status_code >= 500:
                    raise ResponseError(
                        'Server did not respond. {} {}'.format(
                            response.status_code, response.reason))

                raise RequestError('{} {}. Message: {} - {}'.format(
                    response.status_code, response.reason, json_response['code'], json_response['message']))

        return json_response

    # ## ### Actions

    def activate(self):
        return self.request(self.endpoint[:-1], 'POST', params={'action': 'activate'})

    def deactivate(self):
        return self.request(self.endpoint[:-1], 'POST', params={'action': 'deactivate'})

    def upgrade(self):
        return self.request(self.endpoint[:-1], 'POST', params={'action': 'upgrade'})

    def set_members(self):
        # TODO
        pass

    # ## ### API Resources

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/account/
    def account(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/apiKey/
    def api_key(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/auditLog/
    def audit_log(self):
        # TODO check if it's relevant to env
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/certificate/
    def certificate(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/container/
    def container(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/dnsService/
    def dns_service(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/externalService/
    def external_service(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/genericObject/
    def generic_object(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/host/
    def host(self, host_id=None, host_name=None):
        return Host(environment=self, host_id=host_id, host_name=host_name)

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/identity/
    def identity(self):
        # TODO check if it's relevant to env
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/loadBalancerService/
    def load_balancer_service(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/machine/
    def machine(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/machineDriver/
    def machine_driver(self):
        # TODO check if it's relevant to env
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/network/
    def network(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/networkDriver/
    def network_driver(self):
        # TODO check if it's relevant to env
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/networkDriverService/
    def network_driver_service(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/projectMember/
    def project_member(self):
        # TODO check how to implement
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/projectTemplate/
    def project_template(self):
        # TODO check if it's relevant to env
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/pullTask/
    def pull_task(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/registrationToken/
    def registration_token(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/registry/
    def registry(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/registryCredential/
    def registry_credential(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/service/
    def service(self, service_id=None, service_name=None):
        return Service(environment=self, service_id=service_id, service_name=service_name)

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/setting/
    def setting(self):
        # TODO check if it's relevant to env
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/stack/
    def stack(self, stack_id=None, stack_name=None):
        return Stack(environment=self, stack_id=stack_id, stack_name=stack_name)

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/storageDriver/
    def storage_driver(self, storage_driver_id=None, storage_driver_name=None):
        return StorageDriver(environment=self, storage_driver_id=storage_driver_id, storage_driver_name=storage_driver_name)

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/storageDriverService/
    def storage_driver_service(self):
        # TODO
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/subnet/
    def subnet(self):
        # TODO check if it's relevant to env
        pass

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/subscribe/
    def subscribe(self, *args, **kwargs):
        return Subscribe(environment=self, *args, **kwargs)

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/volume/
    def volume(self, volume_id=None, volume_name=None):
        return Volume(environment=self, volume_id=volume_id, volume_name=volume_name)

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/volumeTemplate/
    def volume_template(self):
        # TODO
        pass



from .common import CattleObject
from .exceptions import RequestError


class Stack(CattleObject):
    object_url = 'stacks'

    def __init__(self, environment, stack_id=None, stack_name=None):
        self.env = environment
        if stack_id:
            self.stack_url = '{}/{}'.format(self.object_url, stack_id)
            stack_data = environment.request(self.stack_url, 'GET')
        elif stack_name:
            self.name = stack_name
            stack_data = self.find_by_name()
            if stack_data:
                self.stack_url = '{}/{}'.format(self.object_url, stack_data.get('id'))
        else:
            # Create stack object for list fetch
            stack_data = {}
        # Read only values
        self.state = stack_data.get('state') or None
        self.healthState = stack_data.get('healthState') or None
        self.id = stack_data.get('id') or None
        self.serviceIds = stack_data.get('serviceIds') or []
        self.system = stack_data.get('system') or False
        # Write values
        self.binding = stack_data.get('binding') or {}
        self.description = stack_data.get('description') or None
        self.dockerCompose = stack_data.get('dockerCompose') or ''
        self.environment = stack_data.get('environment') or {}
        self.externalId = stack_data.get('externalId') or None
        self.group = stack_data.get('group') or None
        self.name = stack_data.get('name') or stack_name
        self.outputs = stack_data.get('outputs') or {}
        self.previousEnvironment = stack_data.get('previousEnvironment') or {}
        self.previousExternalId = stack_data.get('previousExternalId') or None
        self.rancherCompose = stack_data.get('rancherCompose') or ''
        self.startOnCreate = stack_data.get('startOnCreate') or False

    @property
    def get_metadata(self):
        if not self.id:
            raise RequestError
        json = {
            'id': self.id,
            'name': self.name,
            'url': self.env.endpoint + self.stack_url,
            'state': self.state,
            'healthState': self.healthState,
            'serviceIds': self.serviceIds,
            'system': self.system,
            'binding': self.binding,
            'description': self.description,
            'dockerCompose': self.dockerCompose,
            'env': self.environment,
            'externalId': self.externalId,
            'group': self.group,
            'outputs': self.outputs,
            'previousEnvironment': self.previousEnvironment,
            'previousExternalId': self.previousExternalId,
            'rancherCompose': self.rancherCompose,
            'startOnCreate': self.startOnCreate
        }
        return json

    def create(self):
        data = {
            'binding': self.binding,
            'description': self.description,
            'dockerCompose': self.dockerCompose,
            'env': self.environment,
            'externalId': self.externalId,
            'group': self.group,
            'name': self.name,
            'outputs': self.outputs,
            'previousEnvironment': self.previousEnvironment,
            'previousExternalId': self.previousExternalId,
            'rancherCompose': self.rancherCompose,
            'startOnCreate': self.startOnCreate
        }
        new_stack = self.env.request(self.object_url, 'POST', data=data)
        self.state = new_stack.get('state')
        self.healthState = new_stack.get('healthState')
        self.id = new_stack.get('id')
        self.serviceIds = new_stack.get('serviceIds')
        self.system = new_stack.get('system')
        self.stack_url = '{}/{}'.format(self.object_url, self.id)

    def save(self):
        if not self.id:
            raise RequestError
        data = {
            'binding': self.binding,
            'description': self.description,
            'externalId': self.externalId,
            'group': self.group,
            'name': self.name,
            'outputs': self.outputs,
            'previousEnvironment': self.previousEnvironment,
            'previousExternalId': self.previousExternalId
        }
        self.env.request(self.stack_url, 'PUT', data=data)

    # Actions

    def activate_services(self):
        return self.action('activateservices')

    def deactivate_services(self):
        return self.action('deactivateservices')

    def cancel_upgrade(self):
        return self.action('cancelupgrade')

    def finish_upgrade(self):
        return self.action('finishupgrade')

    def rollback(self):
        return self.action('rollback')

    def error(self):
        return self.action('error')

    def add_outputs(self, outputs):
        self.action('addoutputs', data=outputs)
        self.outputs.update(outputs)

    def export_config(self, service_ids):
        return self.action('exportconfig', data=service_ids)

    def upgrade(self):
        data = {
            'dockerCompose': self.dockerCompose,
            'environment': self.environment,
            'externalId': self.externalId,
            'rancherCompose': self.rancherCompose
        }
        self.action('upgrade', data=data)



from .common import CattleObject
from .exceptions import RequestError


class Service(CattleObject):
    object_url = 'services'

    def __init__(self, environment, service_id=None, service_name=None):
        self.env = environment
        if service_id:
            self.service_url = '{}/{}'.format(self.object_url, service_id)
            service_data = environment.request(self.service_url, 'GET')
        elif service_name:
            self.name = service_name
            service_data = self.find_by_name()
            if service_data:
                self.service_url = '{}/{}'.format(self.object_url, service_data.get('id'))
        else:
            # Create service object for list fetch
            service_data = {}

        # Read only values
        self.state = service_data.get('state') or None
        self.healthState = service_data.get('healthState') or None
        self.id = service_data.get('id') or None
        self.instanceIds = service_data.get('instanceIds') or []
        self.system = service_data.get('system') or False
        self.createIndex = service_data.get('createIndex') or None
        self.currentScale = service_data.get('currentScale') or None
        self.fqdn = service_data.get('fqdn') or None
        self.linkedServices = service_data.get('linkedServices') or {}
        self.publicEndpoints = service_data.get('publicEndpoints') or []
        self.upgrade = service_data.get('upgrade') or {}
        # Write values
        self.assignServiceIpAddress = service_data.get('assignServiceIpAddress') or False
        self.description = service_data.get('description') or None
        self.externalId = service_data.get('externalId') or None
        self.launchConfig = service_data.get('launchConfig') or {}
        self.lbConfig = service_data.get('lbConfig') or {}
        self.metadata = service_data.get('metadata') or {}
        self.name = service_data.get('name') or service_name
        self.retainIp = service_data.get('retainIp') or False
        self.scale = service_data.get('scale') or 1
        self.scalePolicy = service_data.get('scalePolicy') or {}
        self.secondaryLaunchConfigs = service_data.get('secondaryLaunchConfigs') or []
        self.selectorContainer = service_data.get('selectorContainer') or None
        self.selectorLink = service_data.get('selectorLink') or None
        self.stackId = service_data.get('stackId') or {}
        self.startOnCreate = service_data.get('startOnCreate') or False
        self.vip = service_data.get('vip') or None

    @property
    def get_metadata(self):
        if not self.id:
            raise RequestError
        json = {
            'id': self.id,
            'name': self.name,
            'state': self.state,
            'healthState': self.healthState,
            'instanceIds': self.instanceIds,
            'system': self.system,
            'createIndex': self.createIndex,
            'currentScale': self.currentScale,
            'fqdn': self.fqdn,
            'linkedServices': self.linkedServices,
            'publicEndpoints': self.publicEndpoints,
            'upgrade': self.upgrade,
            'assignServiceIpAddress': self.assignServiceIpAddress,
            'description': self.description,
            'externalId': self.externalId,
            'launchConfig': self.launchConfig,
            'lbConfig': self.lbConfig,
            'metadata': self.metadata,
            'retainIp': self.retainIp,
            'scale': self.scale,
            'scalePolicy': self.scalePolicy,
            'secondaryLaunchConfigs': self.secondaryLaunchConfigs,
            'selectorContainer': self.selectorContainer,
            'selectorLink': self.selectorLink,
            'stackId': self.stackId,
            'startOnCreate': self.startOnCreate,
            'vip': self.vip,
        }
        return json

    def create(self):
        data = {
            'assignServiceIpAddress': self.assignServiceIpAddress,
            'description': self.description,
            'externalId': self.externalId,
            'launchConfig': self.launchConfig,
            'lbConfig': self.lbConfig,
            'metadata': self.metadata,
            'name': self.name,
            'retainIp': self.retainIp,
            'scale': self.scale,
            'scalePolicy': self.scalePolicy,
            'secondaryLaunchConfigs': self.secondaryLaunchConfigs,
            'selectorContainer': self.selectorContainer,
            'selectorLink': self.selectorLink,
            'stackId': self.stackId,
            'startOnCreate': self.startOnCreate,
            'vip': self.vip,
        }
        new_stack = self.env.request(self.object_url, 'POST', data=data)
        self.state = new_stack.get('state')
        self.healthState = new_stack.get('healthState')
        self.id = new_stack.get('id')
        self.instanceIds = new_stack.get('instanceIds')
        self.system = new_stack.get('system')
        self.createIndex = new_stack.get('createIndex')
        self.currentScale = new_stack.get('currentScale')
        self.fqdn = new_stack.get('fqdn')
        self.linkedServices = new_stack.get('linkedServices')
        self.publicEndpoints = new_stack.get('publicEndpoints')
        self.upgrade = new_stack.get('upgrade')
        self.service_url = '{}/{}'.format(self.object_url, self.id)
        return True

    def save(self):
        data = {
            "description": self.description,
            "lbConfig": self.lbConfig,
            "metadata": self.metadata,
            "name": self.name,
            "retainIp": self.retainIp,
            "scale": self.scale,
            "scalePolicy": self.scalePolicy,
            "selectorContainer": self.selectorContainer,
            "selectorLink": self.selectorLink
        }
        return self.env.request(self.service_url, 'PUT', data=data)

    def scale_in(self, scale):
        self.scale -= scale
        if self.scale < 0:
            self.scale += scale
            raise RequestError('Scale can\'t be lower than 0, requested {}'.format(self.scale - scale))
        return self.env.request(self.service_url, 'PUT', {"scale": self.scale})

    def scale_out(self, scale):
        self.scale += scale
        return self.env.request(self.service_url, 'PUT', {"scale": self.scale})

    def scale_to(self, scale):
        if scale < 0:
            raise RequestError('Scale can\'t be lower than 0, requested {}'.format(self.scale - scale))
        self.scale = scale
        return self.env.request(self.service_url, 'PUT', {"scale": self.scale})

    # http://docs.rancher.com/rancher/v1.3/en/api/v2-beta/api-resources/serviceLog/
    def logs(self):
        return self.env.request('{}/servicelogs'.format(self.service_url), 'GET')

    # Actions

    def activate(self):
        return self.action('activate')

    def cancel_upgrade(self):
        return self.action('cancelupgrade')

    def continue_upgrade(self):
        return self.action('continueupgrade')

    def deactivate(self):
        return self.action('deactivate')

    def finish_upgrade(self):
        return self.action('finishupgrade')

    def rollback(self):
        return self.action('rollback')

    def add_service_link(self, service_link):
        return self.action('addservicelink', data=service_link)

    def remove_service_link(self, service_link):
        return self.action('removeservicelink', data=service_link)

    def restart(self, rolling_restart_strategy):
        return self.action('restart', data=rolling_restart_strategy)

    def set_service_links(self, service_links):
        return self.action('setservicelinks', data=service_links)

    def service_upgrade(self, in_service_strategy, to_service_strategy):
        return self.action('upgrade', data={'inServiceStrategy': in_service_strategy,
                                            'toServiceStrategy': to_service_strategy})



class APIException(Exception):
    pass


class JSONDecodeError(APIException):
    pass


class NoTokenProvided(APIException):
    pass


class ResponseError(APIException):
    pass


class RequestError(APIException):
    pass


from .common import CattleObject
from .exceptions import RequestError


class StorageDriver(CattleObject):
    object_url = 'storagedrivers'

    def __init__(self, environment, storage_driver_id=None, storage_driver_name=None):
        self.env = environment
        if storage_driver_id:
            self.storage_driver_url = '{}/{}'.format(self.object_url, storage_driver_id)
            storage_driver_data = environment.request(self.storage_driver_url, 'GET')
        elif storage_driver_name:
            self.name = storage_driver_name
            storage_driver_data = self.find_by_name()
            if storage_driver_data:
                self.storage_driver_url = '{}/{}'.format(self.object_url, storage_driver_data.get('id'))
        else:
            # Create stack object for list fetch
            storage_driver_data = {}
        # Read only values
        self.id = storage_driver_data.get('id') or None
        self.type = storage_driver_data.get('type') or ''
        self.state = storage_driver_data.get('state') or None
        self.serviceId = storage_driver_data.get('serviceId') or None

        # Write values
        self.name = storage_driver_data.get('name') or {}
        self.blockDevicePath = storage_driver_data.get('blockDevicePath') or ''
        self.description = storage_driver_data.get('description') or None
        self.scope = storage_driver_data.get('scope') or None


    @property
    def get_metadata(self):
        if not self.id:
            raise RequestError
        json = {
            'id': self.id,
            'name': self.name,
            'url': self.env.endpoint + self.storage_driver_url,
            'state': self.state,
            'serviceId': self.serviceId,
            'type': self.type,
            'blockDevicePath': self.blockDevicePath,
            'description': self.description,
            'scope': self.scope
        }
        return json

    # Actions

    def update(self):
        return self.action('update')

    def remove(self):
        return self.action('remove')

    def deactivate(self):
        return self.action('deactivate')



from .common import CattleObject
from .exceptions import RequestError


class Host(CattleObject):
    object_url = 'hosts'

    def __init__(self, environment, host_id=None, host_name=None):
        self.env = environment
        if host_id:
            self.host_url = '{}/{}'.format(self.object_url, host_id)
            host_data = environment.request(self.host_url, 'GET')
        elif host_name:
            self.name = host_name
            host_data = self.find_by_name()
            if host_data:
                self.host_url = '{}/{}'.format(self.object_url, host_data.get('id'))
        else:
            # Create host object for list fetch
            host_data = {}
        # Read only values
        self.state = host_data.get('state') or None
        self.id = host_data.get('id') or host_id
        self.name = host_data.get('name') or None
        # Write values
        self.hostname = host_data.get('hostname') or host_name

    @property
    def get_metadata(self):
        if not self.id:
            raise RequestError
        json = {
            'id': self.id,
            'name': self.name,
            'hostname': self.hostname,
            'url': self.env.endpoint + self.host_url,
            'state': self.state
        }
        return json

    def create(self):
        data = {
            'hostname': self.hostname
        }
        new_host = self.env.request(self.object_url, 'POST', data=data)
        self.state = new_host.get('state')
        self.id = new_host.get('id')
        self.host_url = '{}/{}'.format(self.object_url, self.id)

    def save(self):
        if not self.id:
            raise RequestError
        data = {
            'hostname': self.hostname,
        }
        self.env.request(self.host_url, 'PUT', data=data)

    # Actions

    def activate(self):
        return self.action('activate')

    def deactivate(self):
        return self.action('deactivate')

    def delete(self):
        return self.action('delete')

    def error(self):
        return self.action('error')

    def upgrade(self):
        data = {
            'hostname': self.hostname
        }
        self.action('upgrade', data=data)



from base64 import b64encode
try:
    # Python 3
    from urllib.parse import urljoin
except (ImportError) as e:
    # Python 2
    from urlparse import urljoin

import websocket

from .common import CattleObject


class Subscribe(CattleObject):
    object_url = 'subscribe'

    def __init__(self, environment, event_names='resource.change', on_open = None, on_message = None, on_error = None, on_close = None):
        self.env = environment
        self.event_names = event_names
        self.service_url = '{}?eventNames={}'.format(self.object_url, self.event_names)

        # Callbacks
        self.on_open = on_open
        self.on_message = on_message
        self.on_error = on_error
        self.on_close = on_close

        # Websocket
        url = urljoin(self.env.endpoint, self.service_url)
        url = url.replace('http://', 'ws://').replace('https://', 'wss://')
        self.ws = websocket.WebSocketApp(url,
                                         header=["Authorization: Basic " + b64encode(b':'.join(self.env.auth)).strip()],
                                         on_message = self.on_message,
                                         on_error = self.on_error,
                                         on_close = self.on_close,
                                         on_open=self.on_open)

    def connect(self, ping_interval=5):
        self.ws.run_forever(ping_interval=ping_interval)

    def disconnect(self):
        self.ws.close()



import json

from . import RequestError


class CattleObject(object):
    object_url = None

    def __call__(self):
        print(json.dumps(self.__dict__, indent=4))

    def find_by_name(self):
        cattle_objects = self.env.request(self.object_url, 'GET')
        return next((cattle_object for cattle_object in cattle_objects['data'] if cattle_object['name'] == self.name),
                    {})

    def all(self, params=None):
        response = self.env.request(self.object_url, 'GET', params=params)
        return response.get('data')

    def delete(self):
        return self.env.request(self.object_url, 'DELETE')

    def action(self, action, data=None):
        if not self.id:
            raise RequestError
        return self.env.request('{}/{}'.format(self.object_url, self.id), 'POST', params={'action': action},
                                        data=data)



from setuptools import setup
from setuptools import find_packages

with open('requirements.txt') as f:
    required = f.read().splitlines()


def get_version():
    with open("pyranch/__init__.py") as f:
        for line in f:
            if line.startswith("__version__"):
                return eval(line.split("=")[-1])


with open("README.md", "r") as readme_file:
    readme = readme_file.read()

setup(
    name = 'pyranch',
    version = get_version(),
    packages = find_packages(),
    author = 'Federico Torresan',
    author_email = 'federico.torresan@quentral.com',
    url = 'https://github.com/fedetorre/pyranch',
    license = 'LGPLv3',
    install_requires=required,
    description="Rancher API SDK",
    long_description = readme,
    long_description_content_type="text/markdown",
    classifiers = [
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ]
)



from .common import CattleObject
from .exceptions import RequestError


class Volume(CattleObject):
    object_url = 'volumes'

    def __init__(self, environment, volume_id=None, volume_name=None):
        self.env = environment
        if volume_id:
            self.volume_url = '{}/{}'.format(self.object_url, volume_id)
            volume_data = environment.request(self.volume_url, 'GET')
        elif volume_name:
            self.name = volume_name
            volume_data = self.find_by_name()
            if volume_data:
                self.volume_url = '{}/{}'.format(self.object_url, volume_data.get('id'))
        else:
            # Create stack object for list fetch
            volume_data = {}

        # Read only values
        self.id = volume_data.get('id') or None
        self.state = volume_data.get('state') or None

        # Write values
        self.name = volume_data.get('name') or volume_name
        self.description = volume_data.get('description') or None
        self.storageDriverId = volume_data.get('storageDriverId') or None
        self.driver = volume_data.get('driver') or None
        self.driverOpts = volume_data.get('driverOpts') or {}
        self.externalId = volume_data.get('externalId') or None
        self.hostId = volume_data.get('hostId') or None
        self.imageId = volume_data.get('imageId') or None
        self.instanceId = volume_data.get('instanceId') or None
        self.stackId = volume_data.get('stackId') or None
        self.sizeMb = volume_data.get('sizeMb') or None
        self.volumeTemplateId = volume_data.get('volumeTemplateId') or None

    @property
    def get_metadata(self):
        if not self.id:
            raise RequestError
        json = {
            'id': self.id,
            'name': self.name,
            'url': self.env.endpoint + self.volume_url,
            'state': self.state,
            'description': self.description,
            'storageDriverId': self.storageDriverId,
            'driver': self.driver,
            'driverOpts': self.driverOpts,
            'externalId': self.externalId,
            'hostId': self.hostId,
            'imageId': self.imageId,
            'instanceId': self.instanceId,
            'stackId': self.stackId
        }
        return json

    def create(self):
        data = {
            'name': self.name,
            'description': self.description,
            'hostId': self.hostId,
            'storageDriverId': self.storageDriverId,
            'driver': self.driver,
            'driverOpts': self.driverOpts,
            'externalId': self.externalId,
            'imageId': self.imageId,
            'instanceId': self.instanceId,
            'stackId': self.stackId,
            'sizeMb': self.sizeMb,
            'volumeTemplateId': self.volumeTemplateId
        }
        new_volume = self.env.request(self.object_url, 'POST', data=data)
        self.state = new_volume.get('state')
        self.id = new_volume.get('id')
        self.volume_url = '{}/{}'.format(self.object_url, self.id)

    def save(self):
        if not self.id:
            raise RequestError
        data = {
            'name': self.name,
            'description': self.description,
            'hostId': self.hostId,
            'storageDriverId': self.storageDriverId,
            'driver': self.driver,
            'driverOpts': self.driverOpts,
            'externalId': self.externalId,
            'imageId': self.imageId,
            'instanceId': self.instanceId,
            'stackId': self.stackId,
            'sizeMb': self.sizeMb,
            'volumeTemplateId': self.volumeTemplateId
        }
        self.env.request(self.volume_url, 'PUT', data=data)

    # Actions

    def allocate(self):
        return self.action('allocate')

    def deallocate(self):
        return self.action('deallocate')

    def update(self):
        return self.action('update')

    def purge(self):
        return self.action('purge')

    def remove(self):
        return self.action('remove')

    def restorefrombackup(self):
        return self.action('restorefrombackup')

    def reverttosnapshot(self):
        return self.action('reverttosnapshot')

    def snapshot(self):
        return self.action('snapshot')